from typing import Dict, Tuple

from difflib import SequenceMatcher

_COMBATS_RAW: Dict[int, Tuple[str, str]] = {
    1: ('BASE_HP', 'BASE_HP'), 
    2: ('FIGHT_PROP_HP', 'HP'), 
    3: ('FIGHT_PROP_HP_PERCENT', 'HP_PERCENT'), 
    4: ('FIGHT_PROP_BASE_ATTACK', 'BASE_ATTACK'), 
    5: ('FIGHT_PROP_ATTACK', 'ATTACK'), 
    6: ('FIGHT_PROP_ATTACK_PERCENT', 'ATTACK_PERCENT'), 
    7: ('FIGHT_PROP_BASE_DEFENSE', 'BASE_DEFENSE'), 
    8: ('FIGHT_PROP_DEFENSE', 'DEFENSE'), 
    9: ('FIGHT_PROP_DEFENSE_PERCENT', 'DEFENSE_PERCENT'), 
    10: ('FIGHT_PROP_BASE_SPEED', 'BASE_SPEED'), 
    11: ('FIGHT_PROP_SPEED_PERCENT', 'SPEED_PERCENT'), 
    12: ('FIGHT_PROP_HP_MP_PERCENT', 'HP_MP_PERCENT'), 
    13: ('FIGHT_PROP_ATTACK_MP_PERCENT', 'ATTACK_MP_PERCENT'), 
    20: ('FIGHT_PROP_CRITICAL', 'CRITICAL'), 
    21: ('FIGHT_PROP_ANTI_CRITICAL', 'ANTI_CRITICAL'), 
    22: ('FIGHT_PROP_CRITICAL_HURT', 'CRITICAL_HURT'), 
    23: ('FIGHT_PROP_CHARGE_EFFICIENCY', 'CHARGE_EFFICIENCY'), 
    24: ('FIGHT_PROP_ADD_HURT', 'ADD_HURT'), 
    25: ('FIGHT_PROP_SUB_HURT', 'SUB_HURT'), 
    26: ('FIGHT_PROP_HEAL_ADD', 'HEAL_ADD'), 
    27: ('FIGHT_PROP_HEALED_ADD', 'HEALED_ADD'), 
    28: ('FIGHT_PROP_ELEMENT_MASTERY', 'ELEMENT_MASTERY'), 
    29: ('FIGHT_PROP_PHYSICAL_SUB_HURT', 'PHYSICAL_SUB_HURT'), 
    30: ('FIGHT_PROP_PHYSICAL_ADD_HURT', 'PHYSICAL_ADD_HURT'), 
    31: ('FIGHT_PROP_DEFENCE_IGNORE_RATIO', 'DEFENCE_IGNORE_RATIO'), 
    32: ('FIGHT_PROP_DEFENCE_IGNORE_DELTA', 'DEFENCE_IGNORE_DELTA'), 
    40: ('FIGHT_PROP_FIRE_ADD_HURT', 'FIRE_ADD_HURT'), 
    41: ('FIGHT_PROP_ELEC_ADD_HURT', 'ELEC_ADD_HURT'), 
    42: ('FIGHT_PROP_WATER_ADD_HURT', 'WATER_ADD_HURT'), 
    43: ('FIGHT_PROP_GRASS_ADD_HURT', 'GRASS_ADD_HURT'), 
    44: ('FIGHT_PROP_WIND_ADD_HURT', 'WIND_ADD_HURT'), 
    45: ('FIGHT_PROP_ROCK_ADD_HURT', 'ROCK_ADD_HURT'), 
    46: ('FIGHT_PROP_ICE_ADD_HURT', 'ICE_ADD_HURT'), 
    47: ('FIGHT_PROP_HIT_HEAD_ADD_HURT', 'HIT_HEAD_ADD_HURT'), 
    50: ('FIGHT_PROP_FIRE_SUB_HURT', 'FIRE_SUB_HURT'), 
    51: ('FIGHT_PROP_ELEC_SUB_HURT', 'ELEC_SUB_HURT'), 
    52: ('FIGHT_PROP_WATER_SUB_HURT', 'WATER_SUB_HURT'), 
    53: ('FIGHT_PROP_GRASS_SUB_HURT', 'GRASS_SUB_HURT'), 
    54: ('FIGHT_PROP_WIND_SUB_HURT', 'WIND_SUB_HURT'),
    55: ('FIGHT_PROP_ROCK_SUB_HURT', 'ROCK_SUB_HURT'), 
    56: ('FIGHT_PROP_ICE_SUB_HURT', 'ICE_SUB_HURT'), 
    60: ('FIGHT_PROP_EFFECT_HIT', 'EFFECT_HIT'), 
    61: ('FIGHT_PROP_EFFECT_RESIST', 'EFFECT_RESIST'), 
    62: ('FIGHT_PROP_FREEZE_RESIST', 'FREEZE_RESIST'), 
    63: ('FIGHT_PROP_TORPOR_RESIST', 'TORPOR_RESIST'), 
    64: ('FIGHT_PROP_DIZZY_RESIST', 'DIZZY_RESIST'), 
    65: ('FIGHT_PROP_FREEZE_SHORTEN', 'FREEZE_SHORTEN'), 
    66: ('FIGHT_PROP_TORPOR_SHORTEN', 'TORPOR_SHORTEN'), 
    67: ('FIGHT_PROP_DIZZY_SHORTEN', 'DIZZY_SHORTEN'), 
    70: ('FIGHT_PROP_MAX_FIRE_ENERGY', 'MAX_FIRE_ENERGY'), 
    71: ('FIGHT_PROP_MAX_ELEC_ENERGY', 'MAX_ELEC_ENERGY'), 
    72: ('FIGHT_PROP_MAX_WATER_ENERGY', 'MAX_WATER_ENERGY'), 
    73: ('FIGHT_PROP_MAX_GRASS_ENERGY', 'MAX_GRASS_ENERGY'), 
    74: ('FIGHT_PROP_MAX_WIND_ENERGY', 'MAX_WIND_ENERGY'), 
    75: ('FIGHT_PROP_MAX_ICE_ENERGY', 'MAX_ICE_ENERGY'), 
    76: ('FIGHT_PROP_MAX_ROCK_ENERGY', 'MAX_ROCK_ENERGY'), 
    80: ('FIGHT_PROP_SKILL_CD_MINUS_RATIO', 'SKILL_CD_MINUS_RATIO'), 
    81: ('FIGHT_PROP_SHIELD_COST_MINUS_RATIO', 'SHIELD_COST_MINUS_RATIO'), 
    1000: ('FIGHT_PROP_CUR_FIRE_ENERGY', 'CUR_FIRE_ENERGY'),
    1001: ('FIGHT_PROP_CUR_ELEC_ENERGY', 'CUR_ELEC_ENERGY'), 
    1002: ('FIGHT_PROP_CUR_WATER_ENERGY', 'CUR_WATER_ENERGY'), 
    1003: ('FIGHT_PROP_CUR_GRASS_ENERGY', 'CUR_GRASS_ENERGY'), 
    1004: ('FIGHT_PROP_CUR_WIND_ENERGY', 'CUR_WIND_ENERGY'), 
    1005: ('FIGHT_PROP_CUR_ICE_ENERGY', 'CUR_ICE_ENERGY'), 
    1006: ('FIGHT_PROP_CUR_ROCK_ENERGY', 'CUR_ROCK_ENERGY'), 
    1010: ('FIGHT_PROP_CUR_HP', 'CUR_HP'), 
    2000: ('FIGHT_PROP_MAX_HP', 'MAX_HP'), 
    2001: ('FIGHT_PROP_CUR_ATTACK', 'CUR_ATTACK'), 
    2002: ('FIGHT_PROP_CUR_DEFENSE', 'CUR_DEFENSE'), 
    2003: ('FIGHT_PROP_CUR_SPEED', 'CUR_SPEED'), 
    3000: ('FIGHT_PROP_NONEXTRA_ATTACK', 'NONEXTRA_ATTACK'), 
    3001: ('FIGHT_PROP_NONEXTRA_DEFENSE', 'NONEXTRA_DEFENSE'), 
    3002: ('FIGHT_PROP_NONEXTRA_CRITICAL', 'NONEXTRA_CRITICAL'), 
    3003: ('FIGHT_PROP_NONEXTRA_ANTI_CRITICAL', 'NONEXTRA_ANTI_CRITICAL'), 
    3004: ('FIGHT_PROP_NONEXTRA_CRITICAL_HURT', 'NONEXTRA_CRITICAL_HURT'), 
    3005: ('FIGHT_PROP_NONEXTRA_CHARGE_EFFICIENCY', 'NONEXTRA_CHARGE_EFFICIENCY'), 
    3006: ('FIGHT_PROP_NONEXTRA_ELEMENT_MASTERY', 'NONEXTRA_ELEMENT_MASTERY'), 
    3007: ('FIGHT_PROP_NONEXTRA_PHYSICAL_SUB_HURT', 'NONEXTRA_PHYSICAL_SUB_HURT'), 
    3008: ('FIGHT_PROP_NONEXTRA_FIRE_ADD_HURT', 'NONEXTRA_FIRE_ADD_HURT'), 
    3009: ('FIGHT_PROP_NONEXTRA_ELEC_ADD_HURT', 'NONEXTRA_ELEC_ADD_HURT'), 
    3010: ('FIGHT_PROP_NONEXTRA_WATER_ADD_HURT', 'NONEXTRA_WATER_ADD_HURT'), 
    3011: ('FIGHT_PROP_NONEXTRA_GRASS_ADD_HURT', 'NONEXTRA_GRASS_ADD_HURT'), 
    3012: ('FIGHT_PROP_NONEXTRA_WIND_ADD_HURT', 'NONEXTRA_WIND_ADD_HURT'), 
    3013: ('FIGHT_PROP_NONEXTRA_ROCK_ADD_HURT', 'NONEXTRA_ROCK_ADD_HURT'), 
    3014: ('FIGHT_PROP_NONEXTRA_ICE_ADD_HURT', 'NONEXTRA_ICE_ADD_HURT'), 
    3015: ('FIGHT_PROP_NONEXTRA_FIRE_SUB_HURT', 'NONEXTRA_FIRE_SUB_HURT'), 
    3016: ('FIGHT_PROP_NONEXTRA_ELEC_SUB_HURT', 'NONEXTRA_ELEC_SUB_HURT'), 
    3017: ('FIGHT_PROP_NONEXTRA_WATER_SUB_HURT', 'NONEXTRA_WATER_SUB_HURT'), 
    3018: ('FIGHT_PROP_NONEXTRA_GRASS_SUB_HURT', 'NONEXTRA_GRASS_SUB_HURT'), 
    3019: ('FIGHT_PROP_NONEXTRA_WIND_SUB_HURT', 'NONEXTRA_WIND_SUB_HURT'), 
    3020: ('FIGHT_PROP_NONEXTRA_ROCK_SUB_HURT', 'NONEXTRA_ROCK_SUB_HURT'), 
    3021: ('FIGHT_PROP_NONEXTRA_ICE_SUB_HURT', 'NONEXTRA_ICE_SUB_HURT'), 
    3022: ('FIGHT_PROP_NONEXTRA_SKILL_CD_MINUS_RATIO', 'NONEXTRA_SKILL_CD_MINUS_RATIO'), 
    3023: ('FIGHT_PROP_NONEXTRA_SHIELD_COST_MINUS_RATIO', 'NONEXTRA_SHIELD_COST_MINUS_RATIO'), 
    3024: ('FIGHT_PROP_NONEXTRA_PHYSICAL_ADD_HURT', 'NONEXTRA_PHYSICAL_ADD_HURT')
}


# class Combat(object):
#     def __init__(self, raw: Dict[str, str]):
#         self.raw = raw

#     def get_fight_prop_key(self, key: str) -> str:
#         pass
